; /*******************************************************************\
; 				S.M.A.
; \*******************************************************************/
; Code by Patrick Pelissier (1999-2000)

; Compilation data
VERSION		EQU	'0.37'	; Version number
TURBO		EQU	1	; 1 -> 15Hz / 2 -> 30 Hz
MOVING		EQU	2	; 1 -> Normal / 2 -> Plus lent (!)
DEBUG		EQU	0	; 0 -> Off / 1 -> On
SMA_LIB		EQU	0	; Always '0' // A cause de fichiers partager avec 'smalib'

	xdef	_main
	xdef	_comment

	include	tios.h
	include os.h		; Add else 92+/89 or 92 data
	
	include genlib.h
	include filelib.h
	include shrnklib.h	
	include pk92lib.h
	
	ifeq	OS
	include graphlib.h
	include userlib.h
	endif

	ifne	OS
	include flib.h
	endif

	include "macros.h"		; Standart macros
	include "data.h"		; Main data
	include "sprite.h"		; Include file for Sprite objects
	include "const.h"		; Cte
	include "internal.asm"		; Internal macros

prog_start:
	ifeq	OS
	include "AntiCras.asm"
	endif
	include "crash.asm"		; crash handler functions

_main:	;bra.s	_main			; Because Vti ne veut + me faire des points d'arret !
	ifeq	OS
		bsr	Anti_crash_protection	; Check for lib calls Ok.
	endif
	; Initialisation of GenLib and GenLib data
	jsr	genlib::init		; Initialisation of GenLib
	bsr	install_crash_recover	; Install crash recover
	ALLOCATE_MEM	a6		; Allocate memory on the stack and set a6
	genlib::PUSH_DSCREEN	d0	; Allocate a DSCREEN on the stack (7688 bytes)
	move.l	d0,Screen_1(a6)		; Save the adress of the first screen 
	move.l	d0,Current_dscreen(a6)	; Select this screen as the current
	jsr	genlib::set_dscreen_int		; Set the interrupt to this Dscreen
	jsr	genlib::set_dscreen_function	; Set the functions to this DScreen
	jsr	genlib::cls			; Clear it
	jsr	genlib::init_dscreen	; Allocate a new DScreen for double buffer
	move.w	d0,DScreen_hd(a6)	; Save the handle and test if it =0
	beq.s	memory_error		; Memory ?
	move.l	a0,Screen_2(a6)		; Save the adresss of the 2nd screen
	clr.w	Select_s(a6)		; Select the 1st screen
	
	; Reset options data
	ifeq	OS			; Fargo or not Fargo ?
		move.b	CALCULATOR,Calc(a6)	; Not Fargo
	endif
	ifne	OS
		move.b	#2,Calc(a6)		; Fargo
	endif
	move.w	#1,speed_level(a6)
	move.l	#('**  '),Option_str+6
	clr.l	Score(a6)		; Set the score to 0

	; Search Rom / Intro / Menu / play
	bsr	search_rom
	bsr	Intro
	bsr	do_menu	
	bsr	exec_world
	
; Memory error is a label you can use in any place to exit the program.
; It will free any handles using genlib::free_hd and the handle of the screen
; It restores the stack and quit the program.
memory_error:
	move.w	DScreen_hd(a6),d0	; Push the handle
	bsr	Free			; Free the handle
	DALLOCATE_MEM	a6		; Desallocate and restore the correct stack :))))
_exit:					; It is here because a6 isn't (?) valid
	jsr	genlib::free_hd		; If no hd, it's still ok.
	bsr	unsinstall_crash_recover
	jmp	genlib::quit		; Quit the genlib and the program.

	include "Ani.asm"		; Animation code and tables
	include "tiles.asm"		; Tiles animation
	include "surface.asm"		; Surface tables
	include "auto_spr.asm"		; Check sprites routines
	include "sprite.asm"		; Sprite(s) code and data (sma, crabe, ring, ...)
	include "object.asm"		; Object table and code (ring, spike, TV, ...)
	include "engine.asm"		; Main engine
	include "routine.asm"		; Some standart routines
	include "world.asm"		; World functions
	include "menu.asm"		; Menu functions
	include "smaintro.asm"		; Introduction
	include "string.asm"		; All strings data

prog_end:
Print_opt	dc.b	0	; 0 -> 2 planes / 4 -> 1 plane
	EVEN
_comment
	dc.b	"S.M.A. v"	; Program's name.
	dc.l	VERSION		; Version string
	dc.b	0		; Null => string
	EVEN

	end			; End of program
