; Crash handler routines
; 
; Whern an error happened,
; If Crash_Flag is set,
;	it will first try to determine if we are in the main loop
; 	If so, if will free the sprite buffer, and restart tjhe main loop.
;	If the crash is called another, then it will display the scrash screen
; Else it dispaly the crash screen
;
; Original routines made by Patrick Davidson for Phoenix 7.0

; List of the Functions:
;	install_crash_recover
;	unsinstall_crash_recover


; Install the crash handler
; Call it before allocate stack frame !
install_crash_recover:
	bclr.b	#2,$600001                      ; Unprotect low memory

	move.l	(a7)+,d1			; Load return adress

	; Install crash handler
	lea	crash_handler(pc),a0            ; Install crash handler
        moveq	#8,d0
        lea	($8+$4).w,a1
\loop:		move.l  (a1),-(a7)		; Push old handler on stack
		move.l  a0,(a1)+		; Install new one
		addq.l	#6,a0			; Next int
		dbra    d0,\loop

        move.l  a7,originalsp                   ; Save stack pointer
	move.l	d1,-(a7)			; Restore return adress

	bset.b	#2,$600001			; protect low memory
	rts

;	///////

unsinstall_crash_recover:
	bclr.b	#2,$600001                      ; Unprotect low memory

	move.l	(a7)+,d1			; Load return adress

	lea	($30).w,a1                      ; Uninstall crash handler
	moveq	#8,d0
\loop:		move.l	(sp)+,-(a1)
		dbra	d0,\loop

	move.l	d1,-(a7)			; Restore return adress

	bset.b	#2,$600001			; protect low memory
	rts

crash_handler:					; Crash handlers
        move.w  #3,-(a7)			; Store exception number
        bra.s   crash_handler_main		; Go to main handler
        move.w  #4,-(a7)			; Store exception number
        bra.s   crash_handler_main		; Go to main handler
        move.w  #5,-(a7)			; Store exception number
        bra.s   crash_handler_main		; Go to main handler
        move.w  #6,-(a7)			; Store exception number
        bra.s   crash_handler_main		; Go to main handler
        move.w  #7,-(a7)			; Store exception number
        bra.s   crash_handler_main		; Go to main handler
        move.w  #8,-(a7)			; Store exception number
        bra.s   crash_handler_main		; Go to main handler
        move.w  #9,-(a7)			; Store exception number
        bra.s   crash_handler_main		; Go to main handler
        move.w  #10,-(a7)			; Store exception number
        bra.s   crash_handler_main		; Go to main handler
        move.w  #11,-(a7)			; Store exception number

crash_handler_main:
	move.w	(a7)+,crash_err			; Get the error

	move.w  #$2700,sr			; Stop interrupts
	movem.l a0-a7,-(sp)			; Save registers at crash + 64
	move.l	usp,a0				; Save Usp instead of ssp
	move.l	a0,(7*4)(a7)
	movem.l	d0-d7,-(sp)
	lea	64(a7),a6			; Save SP at crash

	bsr	Clear_LCD_MEM

        move.w  #1,-(sp)        		; Sp -= 2
        jsr     tios::FontSetSys 

        lea     LCD_MEM+3810,a3			;Temporary buffer on screen !

        move.l  2(a6),-(sp)                     ; PC
        move.w  crash_err(pc),-(sp)		; Display error#
        pea	firsterrortext(pc)		; String
        pea	(a3)				; Buffer
        bsr	sprintf				; Sp -= 14
	movea.l	a3,a0
        moveq	#1,d0
        moveq	#1,d1
        bsr	Display_string			; Display

        pea     _main(pc)                       ; Display prog start
        move.w  (a6),-(sp)			; SR
        pea     seconderrortext(pc)		; String
        pea     (a3)				; Buffer
        bsr     sprintf				; Sp -= 14
        lea     (a3),a0
        moveq   #1,d0
        moveq   #9,d1
        bsr	Display_string			; Display

        moveq   #91,d1                          ;"Press ON to exit"
        lea     thirderrortext(pc),a0
        bsr	Display_string			; Display

        lea     30(sp),sp			; Sp+=30

        lea     -64(a6),a4
        moveq   #0,d7                           ;D7 = n
        moveq   #18,d6                          ;D6 = line
\loopshowregs:
		; Create string
		move.l  32(a4),-(sp)                    ; An
		move.w  d7,-(sp)			; n
		move.l  (a4)+,-(sp)                     ; Dn
		move.w  d7,-(sp)			; n
		pea     reglisttext(pc)			; String
		pea     (a3)				; Buffer
		bsr     sprintf
		lea     20(sp),sp			; Correct SP

		; Show string
		move.l	a3,a0
		move.w  d6,d1
		moveq   #1,d0
	        bsr	Display_string			; Display
                
		; Next line for register
		addq.w  #8,d6                           ; Next row
		addq.w  #1,d7				; Next register
		cmp.w   #8,d7
		bne.s   \loopshowregs

	; Show a little stack frame
        move.l  usp,a2
        lea     12(a2),a2
        move.l  -(a2),-(sp)
        move.l  -(a2),-(sp)
        move.l  -(a2),-(sp)
        pea     stacktext(pc)
        pea     (a3)
        bsr     sprintf
        lea     (a3),a0
        moveq   #82,d1
        moveq   #1,d0
        bsr	Display_string			; Display
                        
        move.l  #_exit,2(a6)                    ; Adjust return address
        clr.w   (a6)                            ; SR = 0 (user mode)
	move.l	a6,sp				; Restore stack pointer
	move.l	originalsp(Pc),a0
	move.l	a0,Usp				; Restore User stack pointer
	move.l	#LCD_MEM,d0
	lsr.l	#3,d0
	move.w	d0,($600010)			; restore Screen pointer

\wait_on:
		btst    #1,$60001a
		bne.s   \wait_on
        rte                                     ; Return (to exit code)
      
crash_err	dc.w    0			; Error code
originalsp	dc.l    0			; Original Stack Pointer

firsterrortext:
        dc.b    'Exception %02x at %08lx',0
seconderrortext:
        dc.b    'SR=%04x  _main=%08lx',0
thirderrortext:
        dc.b    'Press [ON] to exit',0
reglisttext:
        dc.b    'D%d=%08lx  A%d=%08lx',0
stacktext:      
        dc.b    '%08lx%08lx%08lx',0        
        EVEN
